#include "Camera.h"

#include <glm/gtc/matrix_transform.hpp>

Camera::Camera()
{
	reset();
}

Camera::~Camera(void)
{
}

void Camera::reset()
{
	mPos = glm::vec3(0.0f, 0.0f, -250.0f);
	mPos2 = glm::vec3(0.0f, 0.0f, 1.0f);
	mUp = glm::vec3(0.0f, 1.0f, 0.0f);
	mNear = 0.1f;
	mFar = 1000.0f;
	mFieldOfView = 1.0f;
	mAspect = 4.0f / 3.0f;
	mOrthogonal = false;
	mLeft = -2.0f;
	mRight = 2.0f;
	mTop = 2.0f;
	mBottom = -2.0f;

	update(0, 0, 0, 0, 0, 0);
	buildProjectionMatrix();
}

void Camera::setOrthogonalBorders(float left, float right, float top, float bottom)
{
	mLeft = left;
	mRight = right;
	mTop = top;
	mBottom = bottom;
}

void Camera::update(float pitch, float yaw, float roll, float zoom, float strafeX, float strafeY) {

	// zoom
	_zoom += zoom;
	glm::mat4 zoomMat = glm::translate(glm::mat4(1), mPos + mPos2 * _zoom);

	// strafe
	_strafe += glm::vec3(strafeX, -strafeY, 0);
	glm::mat4 strafeMat = glm::translate(glm::mat4(1), _strafe);

	// rotation
	_rot = glm::normalize(glm::quat(glm::vec3(-pitch, -yaw, roll)) * _rot);
	glm::mat4 rotationMat = glm::mat4_cast(_rot);

	mViewMatrix = zoomMat * strafeMat * rotationMat;
}

void Camera::buildProjectionMatrix(void)
{
	if (mOrthogonal) {
		mProjectionMatrix = glm::ortho(mLeft, mRight, mBottom, mTop, mNear, mFar);
	}
	else {
		mProjectionMatrix = glm::perspective(glm::radians(mFieldOfView), mAspect, mNear, mFar);
	}
}
