/*
* Copyright (C) 2017
* Computer Graphics Group, The Institute of Computer Graphics and Algorithms, TU Wien
* Written by Tobias Klein <tklein@cg.tuwien.ac.at>
* All rights reserved.
*/

#pragma once

#include <glm/glm.hpp>
#include <glm/gtc/quaternion.hpp> 

#define M_PI 3.14159265358979323846

class Camera
{
public:

	float _zoom = 1;
	glm::vec3 _strafe = glm::vec3(0);
	glm::quat _rot = glm::quat();

	Camera::Camera();

	~Camera(void);

	/**
	Set Camera to default values
	*/
	void reset();

	void buildProjectionMatrix(void);

	void setOrthogonalBorders(float left, float right, float top, float bottom);

	/**
	Update camera by input
	*/
	void update(float pitch, float yaw, float roll, float zoom, float strafeX, float strafeY);



	glm::vec3 getUp() { return mUp; };

	bool isOrthogonal() { return mOrthogonal; };

	float getFieldOfView() { return mFieldOfView; };

	void setOrthogonal(bool orthogonal) { mOrthogonal = orthogonal; buildProjectionMatrix();};

	void setUp(glm::vec3 &up) { mUp = up; };

	void setFieldOfView(float fieldOfView) { mFieldOfView = fieldOfView; };

	void setNearPlane(float nearPlane) { mNear = nearPlane; buildProjectionMatrix();};

	void setFarPlane(float farPlane) { mFar = farPlane; buildProjectionMatrix(); };

	float getNearPlane() { return mNear; };

	float getFarPlane() { return mFar; };

	void setAspect(float aspect) { mAspect = aspect;  buildProjectionMatrix(); };

	const glm::mat4 & getViewMatrix() { return mViewMatrix; };

	const glm::mat4 & getProjectionMatrix() { return mProjectionMatrix; };

	float mFieldOfView;
	float mAspect;
	float mNear;
	float mFar;


private:
	glm::mat4 mViewMatrix;
	glm::mat4 mProjectionMatrix;

	glm::vec3 mUp;
	glm::vec3 mPos;
	glm::vec3 mPos2;
	float mLeft;
	float mRight;
	float mTop;
	float mBottom;
	bool mOrthogonal;
};