/*
* Copyright (C) 2017
* Computer Graphics Group, The Institute of Computer Graphics and Algorithms, TU Wien
* Written by Tobias Klein <tklein@cg.tuwien.ac.at>
* All rights reserved.
*/

#pragma once

#include <vector>
#include <string>
#include <iostream>
#include <glm\glm.hpp>

#include <QProgressBar>


//-------------------------------------------------------------------------------------------------
// Voxel
//-------------------------------------------------------------------------------------------------

class Voxel
{
	public:

		Voxel();
		Voxel(const Voxel &other);
		Voxel(const float value);

		~Voxel();


		// VOXEL VALUE

		void					setValue(const float value);
		const float				getValue() const;


		// OPERATORS

		const bool				operator==(const Voxel &other) const;
		const bool				operator!=(const Voxel &other) const;
		const bool				operator>(const Voxel &other) const;
		const bool				operator>=(const Voxel &other) const;
		const bool				operator<(const Voxel &other) const;
		const bool				operator<=(const Voxel &other) const;

		const Voxel				operator+(const Voxel &other) const;
		const Voxel				operator-(const Voxel &other) const;
		const Voxel				operator*(const float &value) const;
		const Voxel				operator/(const float &value) const;
		
		const Voxel&			operator+=(const Voxel &other);
		const Voxel&			operator-=(const Voxel &other);
		const Voxel&			operator*=(const float &value);
		const Voxel&			operator/=(const float &value);


	private:

		float					m_Value;

};


//-------------------------------------------------------------------------------------------------
// Volume
//-------------------------------------------------------------------------------------------------

class Volume
{

	public:

		Volume();
		Volume(int w, int h, int d);
		~Volume();


		// VOLUME DATA

		const Voxel&			voxel(const int i) const;
		const Voxel				voxel(const int x, const int y, const int z) const;
		const Voxel				voxel(const glm::vec3& v) const;

		/**
		tri-linear interpolates any position inside the volume
		*/
		const float				voxelInterpolate(const float x, const float y, const float z) const;
		const float				voxelInterpolate(const glm::vec3& p) const;
		const Voxel*			voxels() const;

		bool					set(float f, int i);
		bool					set(float f, int x, int y, int z);

		const int				width() const;
		const int				height() const;
		const int				depth() const;

		const int				size() const;

		bool                    loadFromFile(QString filename, QProgressBar * progressBar, std::string type);


	private:

		std::vector<Voxel>		m_Voxels;

		int						m_Width;
		int						m_Height;
		int						m_Depth;

		int						m_Size;

};
