-- Vertex

#extension GL_ARB_explicit_attrib_location : enable

layout(location = 0) in vec3 vertex;

out vec2 texCoord;

void main()
{
	texCoord = vec2(vertex.x, vertex.y) * 0.5 + 0.5;
	gl_Position = vec4(vertex, 1);
}

-- Fragment

uniform sampler2D frontFaces;
uniform sampler2D backFaces;
uniform sampler3D volume;

uniform int renderingMode;
uniform float StepLength = 1/100.0;

in vec2 texCoord;
out vec4 fragColor;

void main()
{
	// ToDo
	
	
	switch(renderingMode)
	{
		case 0: //render front faces
		{
			fragColor = texture(frontFaces, texCoord);
			break;
		}
		
		case 1: //render back faces
		{
			fragColor = texture(backFaces, texCoord);
			break;
		}
		
		case 2: //render volume (MIP)
		{

			vec3 start = texture(frontFaces, texCoord).xyz;
			vec3 end = texture(backFaces, texCoord).xyz;

			vec3 ray = end - start;

			float _max = 0;

			if (ray != vec3(0)){
				for (float t = 0; t <= 1; t+=StepLength){
					_max = max(_max, texture(volume, start + t * ray).r);
				}
			}

			fragColor = vec4(vec3(_max),1);
			break;
		}
		case 3: //render volume (Alpha-Compositing)
		{
			vec3 start = texture(frontFaces, texCoord).xyz;
			vec3 end = texture(backFaces, texCoord).xyz;

			vec3 ray = end - start;

			vec4 C = vec4(0);

			if (ray != vec3(0)){

				for (float t = 0; t <= 1 && C.a < 1; t+=StepLength){
					float density = texture(volume, start + t * ray).r;

					//red to yellow gradient
					//vec4 A = vec4(1, 1-density, 0, density);

					//grey
					vec3 col = vec3(0);
					if (density > 0.05f){
						col = vec3(density);
					}
					vec4 A = vec4(col, density);
				
					C = (1.0 - A.a) * C + A.a * A;
				}
			}

			fragColor = C;
			break;
		}
	}
}